﻿using API.Controllers;
using Dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging.Abstractions;
using Moq;
using Shared;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TestAPI.Extensions;

namespace TestAPI
{
    public class SuccessesUnitTest
    {
        private readonly Mock<ISuccessService<SuccessDTO>> successService;
        public SuccessesUnitTest()
        {
            successService = new Mock<ISuccessService<SuccessDTO>>();
        }


        [Fact]
        public void GetSuccessesListSuccess()
        {
            var successesList = GetSuccessesData();
            successService.Setup(x => x.GetSuccesses(1, 4, 0))
                .Returns(successesList);
            var successesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var successesResult = successesController.GetSuccesses(1, 4, 0);

            if (successesResult is OkObjectResult okObjectResult)
            {
                var valeur = okObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.Equal(GetSuccessesData().ToString(), valeur.ToString());
                Assert.True(successesList.SequenceEqual(valeur as IEnumerable<SuccessDTO>, new SuccessIdEqualityComparer()));

            }

        }

        [Fact]
        public void GetSuccessesListFail()
        {
            successService.Setup(x => x.GetSuccesses(1, 4, 0))
                .Returns(new List<SuccessDTO>());
            var successesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var successesResult = successesController.GetSuccesses(2, 3, 0);

            if (successesResult is StatusCodeResult statusCodeResult && statusCodeResult.StatusCode == 204)

            {

                Assert.IsNotType<OkObjectResult>(successesResult);
            }

        }


        [Fact]
        public void GetSuccessInquiryIdSuccess()
        {
            var successesList = GetSuccessesData();
            successService.Setup(x => x.GetSuccessesByInquiryId(1))
                .Returns(new List<SuccessDTO> { successesList[0], successesList[1] });
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var sucessesResult = SuccessesController.GetSuccessByInquiryId(1);
            if (sucessesResult is OkObjectResult okObjectResult)
            {
                List<SuccessDTO> valeur = okObjectResult.Value as List<SuccessDTO>;

                Assert.NotNull(valeur);
                Assert.Equal(0, valeur[0].UserId);
                Assert.Equal(1, valeur[0].InquiryId);
                Assert.True(valeur[0].IsFinished);
                Assert.Equal(1, valeur[1].UserId);
                Assert.Equal(1, valeur[1].InquiryId);
                Assert.True(valeur[1].IsFinished);


                Assert.Equal(valeur[1].GetHashCode(), successesList[1].GetHashCode());
                Assert.True(valeur[1].Equals(successesList[1]));
                Assert.False(valeur.Equals(new object()));
                Assert.False(valeur.Equals(null));
                Assert.True(valeur.Equals(valeur));
                Assert.IsType<SuccessDTO>(valeur[0]);
                Assert.Contains(valeur[1], successesList);
            }

        }

        [Fact]
        public void GetSuccessInquiryIdFail()
        {
            var successesList = GetSuccessesData();
            successService.Setup(x => x.GetSuccessesByInquiryId(1))
                .Returns(new List<SuccessDTO> { successesList[0], successesList[1] });
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var sucessesResult = SuccessesController.GetSuccessByInquiryId(100);
            if (sucessesResult is NotFoundObjectResult NFObjectResult)
            {
                var valeur = NFObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.IsNotType<SuccessDTO>(valeur);
                Assert.DoesNotContain(valeur, successesList);
            }

        }


        [Fact]
        public void GetSuccessUserIdSuccess()
        {
            var successesList = GetSuccessesData();
            successService.Setup(x => x.GetSuccessesByUserId(2))
                .Returns(new List<SuccessDTO> { successesList[2], successesList[3] });
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var sucessesResult = SuccessesController.GetSuccessByUserId(2);
            if (sucessesResult is OkObjectResult okObjectResult)
            {
                List<SuccessDTO> valeur = okObjectResult.Value as List<SuccessDTO>;

                Assert.NotNull(valeur);
                Assert.Equal(2, valeur[0].UserId);
                Assert.Equal(3, valeur[0].InquiryId);
                Assert.True(valeur[0].IsFinished);
                Assert.Equal(2, valeur[1].UserId);
                Assert.Equal(4, valeur[1].InquiryId);
                Assert.True(valeur[1].IsFinished);


                Assert.Equal(valeur[1].GetHashCode(), successesList[3].GetHashCode());
                Assert.True(valeur[1].Equals(successesList[3]));
                Assert.False(valeur.Equals(new object()));
                Assert.IsType<SuccessDTO>(valeur[0]);
                Assert.Contains(valeur[1], successesList);
            }

        }

        [Fact]
        public void GetSuccessUserIdFail()
        {
            var successesList = GetSuccessesData();
            successService.Setup(x => x.GetSuccessesByUserId(2))
                .Returns(new List<SuccessDTO> { successesList[2], successesList[3] });
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var sucessesResult = SuccessesController.GetSuccessByUserId(200);
            if (sucessesResult is NotFoundObjectResult NFObjectResult)
            {
                var valeur = NFObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.IsNotType<SuccessDTO>(valeur);
                Assert.DoesNotContain(valeur, successesList);
            }

        }



        [Fact]
        public void DeleteSuccessSuccess()
        {
            successService.Setup(x => x.DeleteSuccess(1,1))
                .Returns(true);
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var sucessesResult = SuccessesController.DeleteSuccess(1,1);
            if (sucessesResult is OkObjectResult okObjectResult)
            {
                bool valeur = (bool)okObjectResult.Value;

                Assert.True(valeur);
            }

        }


        [Fact]
        public void DeleteSuccessFail()
        {
            successService.Setup(x => x.DeleteSuccess(1, 1))
                .Returns(true);
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var sucessesResult = SuccessesController.DeleteSuccess(100, 278);
            if (sucessesResult is NotFoundObjectResult NFObjectResult)
            {
                Assert.Null(NFObjectResult.Value);
                Assert.IsNotType<bool>(NFObjectResult.Value);
            }
        }


        [Fact]
        public void CreateSuccessSuccess()
        {
            successService.Setup(x => x.CreateSuccess(8,8,true))
                .Returns(new SuccessDTO(8,8,true));
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var successesResult = SuccessesController.CreateSuccess(new SuccessDTO(8,8,true));
            if (successesResult is CreatedResult createdObjectResult)
            {
                SuccessDTO valeur = createdObjectResult.Value as SuccessDTO;

                Assert.NotNull(valeur);
                Assert.Equal(8, valeur.UserId);
                Assert.Equal(8, valeur.InquiryId);
                Assert.True(valeur.IsFinished);
            }

        }

        [Fact]
        public void CreateSuccessFail()
        {
            successService.Setup(x => x.CreateSuccess(8, 8, true))
                .Returns(new SuccessDTO(8, 8, true));
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var successesResult = SuccessesController.CreateSuccess(new SuccessDTO(882, 818, true));

            if (successesResult is BadRequestResult BDObjectResult)
            {

                Assert.Equal(400, BDObjectResult.StatusCode);
            }
        }



        [Fact]
        public void UpdateUserSuccess()
        {
            successService.Setup(x => x.UpdateSuccess(1,1,new SuccessDTO(1,1,true)))
                .Returns(new SuccessDTO(1, 1, true));
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var successesResult = SuccessesController.UpdateSuccess(1, 1, new SuccessDTO(1, 1, true));
            if (successesResult is OkObjectResult okObjectResult)
            {
                SuccessDTO valeur = okObjectResult.Value as SuccessDTO;

                Assert.NotNull(valeur);
                Assert.Equal(1, valeur.UserId);
                Assert.Equal(1, valeur.InquiryId);
                Assert.True(valeur.IsFinished);
            }

        }

        [Fact]
        public void UpdateUserFail()
        {
            successService.Setup(x => x.UpdateSuccess(1, 1, new SuccessDTO(1, 2, true)))
                .Returns(new SuccessDTO(1, 2, true));
            var SuccessesController = new SuccessesController(successService.Object, new NullLogger<SuccessesController>());

            var successesResult = SuccessesController.UpdateSuccess(1, 1, new SuccessDTO(1, 2, true));

            if (successesResult is BadRequestResult BDObjectResult)
            {

                Assert.Equal(400, BDObjectResult.StatusCode);
            }
        }




        private List<SuccessDTO> GetSuccessesData()
        {
            List<SuccessDTO> successesData = new List<SuccessDTO>(4)
            {
                new (0,1,true),
                new (1,1,true), 
                new (2,3,true),
                new (2,4,true),
            };
            return successesData;
        }
    }
}
